/*
* aureport-scan.c - Extract interesting fields and check for match
* Copyright (c) 2005 Red Hat Inc., Durham, North Carolina.
* All Rights Reserved. 
*
* This software may be freely redistributed and/or modified under the
* terms of the GNU General Public License as published by the Free
* Software Foundation; either version 2, or (at your option) any
* later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING. If not, write to the
* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Authors:
*   Steve Grubb <sgrubb@redhat.com>
*/

#include "config.h"
#include <string.h>
#include <stdio.h>
#include "libaudit.h"
#include "aureport-options.h"
#include "ausearch-parse.h"
#include "ausearch-string.h"
#include "aureport-scan.h"

static void do_summary_total(llist *l);
//static int strmatch(const char *needle, const char *haystack);
//static int user_match(llist *l);
//static int group_match(llist *l);

summary_data sd;

/* This function inits the counters */
void reset_counters(void)
{
	sd.changes = 0UL;
	sd.acct_changes = 0UL;
	sd.good_logins = 0UL;
	sd.bad_logins = 0UL;
	sd.events = 0UL;
	sd.avcs = 0UL;
	sd.failed_syscalls = 0UL;
	sd.fs_watches = 0UL;
	sd.anomalies = 0UL;
	sd.responses = 0UL;
	slist_create(&sd.users);
	slist_create(&sd.terms);
	slist_create(&sd.files);
	slist_create(&sd.hosts);
	slist_create(&sd.exes);
	slist_create(&sd.pids);
}

/* This function inits the counters */
void destroy_counters(void)
{
	sd.changes = 0UL;
	sd.acct_changes = 0UL;
	sd.good_logins = 0UL;
	sd.bad_logins = 0UL;
	sd.events = 0UL;
	sd.avcs = 0UL;
	sd.failed_syscalls = 0UL;
	sd.fs_watches = 0UL;
	sd.anomalies = 0UL;
	sd.responses = 0UL;
	slist_clear(&sd.users);
	slist_clear(&sd.terms);
	slist_clear(&sd.files);
	slist_clear(&sd.hosts);
	slist_clear(&sd.exes);
	slist_clear(&sd.pids);
}

/*
 * This function performs that matching of search params with the record.
 * It returns 1 on a match, and 0 if no match.
 */
int scan(llist *l)
{
	// Are we within time range?
	if (start_time == 0 || l->e.sec >= start_time) {
		if (end_time == 0 || l->e.sec <= end_time) {
			if (event_id == -1 || event_id == l->e.serial) {

				// OK - do the heavier checking
				if (extract_search_items(l) == 0) 
					return 1;
			}
		}
	}
	return 0;
}

int per_event_processing(llist *l)
{
	int rc = 0;

	switch (report_type)
	{
		case RPT_SUMMARY:
			do_summary_total(l);
			rc = 1;
			break;
		case RPT_AVC:
			list_first(l);
			if (list_find_msg(l, AUDIT_AVC)) {
				print_per_event_item(l);
				rc = 1;
			}
			break;
		case RPT_CONFIG:
			list_first(l);
			if (list_find_msg(l, AUDIT_CONFIG_CHANGE)) {
				print_per_event_item(l);
				rc = 1;
			}
			break;
		case RPT_LOGIN:
			list_first(l);
			if (list_find_msg(l, AUDIT_USER_LOGIN)) {
				print_per_event_item(l);
				rc = 1;
			}
			break;
		case RPT_ACCT_MOD:
			// FIXME: consider adding/deleting users & groups
			list_first(l);
			if (list_find_msg(l, AUDIT_USER_CHAUTHTOK)) {
				print_per_event_item(l);
				rc = 1;
			}
			break;
		case RPT_EVENT:
			list_first(l);
			if (report_detail == D_DETAILED) {
				print_per_event_item(l);
				rc = 1;
			} else { //  specific event report
				UNIMPLEMENTED;
			}
			break;
		case RPT_FILE:
			list_first(l);
			if (report_detail == D_DETAILED) {
				if (l->s.filename) {
					print_per_event_item(l);
					rc = 1;
				}
			} else { //  specific file report
				UNIMPLEMENTED;
			}
			break;
		case RPT_HOST:
			list_first(l);
			if (report_detail == D_DETAILED) {
				if (l->s.hostname) {
					print_per_event_item(l);
					rc = 1;
				}
			} else { //  specific host report
				UNIMPLEMENTED;
			}
			break;
		case RPT_PID:
			list_first(l);
			if (report_detail == D_DETAILED) {
				if (l->s.pid >= 0) {
					print_per_event_item(l);
					rc = 1;
				}
			} else { //  specific pid report
				UNIMPLEMENTED;
			}
			break;
		case RPT_SYSCALL:
			list_first(l);
			if (report_detail == D_DETAILED) {
				if (l->s.syscall) {
					print_per_event_item(l);
					rc = 1;
				}
			} else { //  specific syscall report
				UNIMPLEMENTED;
			}
			break;
		case RPT_TERM:
			list_first(l);
			if (report_detail == D_DETAILED) {
				if (l->s.terminal) {
					print_per_event_item(l);
					rc = 1;
				}
			} else { //  specific terminal report
				UNIMPLEMENTED;
			}
			break;
		case RPT_USER:
			list_first(l);
			if (report_detail == D_DETAILED) {
				if (l->s.uid != -1) {
					print_per_event_item(l);
					rc = 1;
				}
			} else { //  specific user report
				UNIMPLEMENTED;
			}
			break;
		case RPT_EXE:
			list_first(l);
			if (report_detail == D_DETAILED) {
				if (l->s.exe) {
					print_per_event_item(l);
					rc = 1;
				}
			} else { //  specific exe report
				UNIMPLEMENTED;
			}
			break;
		case RPT_WATCH:
			list_first(l);
			if (report_detail == D_DETAILED) {
				if (list_find_msg(l, AUDIT_FS_WATCH)) {
					print_per_event_item(l);
					rc = 1;
				}
			} else { //  specific watch report
				UNIMPLEMENTED;
			}
			break;
		case RPT_ANOMALY:
			list_first(l);
			if (report_detail == D_DETAILED) {
				if (list_find_msg_range(l, 
						AUDIT_FIRST_ANOM_MSG,
						AUDIT_LAST_ANOM_MSG)) {
					print_per_event_item(l);
					rc = 1;
				}
			} else { // FIXME: specific anom report
				UNIMPLEMENTED;
			}
			break;
		case RPT_RESPONSE:
			list_first(l);
			if (report_detail == D_DETAILED) {
				if (list_find_msg_range(l,	
						AUDIT_FIRST_ANOM_RESP,
						AUDIT_LAST_ANOM_RESP)) {
					print_per_event_item(l);
					rc = 1;
				}
			} else { // FIXME: specific resp report
				UNIMPLEMENTED;
			}
			break;
		default:
			break;
	}
	return rc;
}

static void do_summary_total(llist *l)
{
	char tmp[32];

	// add events
	sd.events++;

	// add config changes
	list_first(l);
	if (list_find_msg(l, AUDIT_CONFIG_CHANGE))
		sd.changes++;

	// add acct changes
	list_first(l);
	if (list_find_msg(l, AUDIT_USER_CHAUTHTOK))
		sd.acct_changes++;
	// FIXME: include: new/del accts & new/del groups

	// add logins
	list_first(l);
	if (list_find_msg(l, AUDIT_USER_LOGIN)) {
		if (l->s.success == 1)
			sd.good_logins++;
		else if (l->s.success == 0)
			sd.bad_logins++;
	}

	// add users
	if (l->s.loginuid != -1) {
		snprintf(tmp, sizeof(tmp), "%d", l->s.loginuid);
		add_if_uniq(&sd.users, tmp);
	}

	// add terminals
	if (l->s.terminal)
		add_if_uniq(&sd.terms, l->s.terminal);

	// add hosts
	if (l->s.hostname)
		add_if_uniq(&sd.hosts, l->s.hostname);

	// add execs
	if (l->s.exe)
		add_if_uniq(&sd.exes, l->s.exe);

	// add files
	if (l->s.filename) {
		const snode *sn;
		slist *sptr = l->s.filename;

		slist_first(sptr);
		sn=slist_get_cur(sptr);
		while (sn) {
			if (sn->str)
				add_if_uniq(&sd.files, sn->str);
			sn=slist_next(sptr);
		} 
	}

	// add avcs
	list_first(l);
	if (list_find_msg(l, AUDIT_AVC)) 
		sd.avcs++;		
	list_first(l);
	if (list_find_msg(l, AUDIT_USER_AVC))
		sd.avcs++;		

	// add failed syscalls
	if (l->s.success == 0 && l->s.syscall > 0)
		sd.failed_syscalls++;

	// add watches
	list_first(l);
	if (list_find_msg(l, AUDIT_FS_WATCH))
		sd.fs_watches++;

	// add pids
	if (l->s.pid != -1) {
		snprintf(tmp, sizeof(tmp), "%d", l->s.pid);
		add_if_uniq(&sd.pids, tmp);
	}

	// add anomalies
	if (list_find_msg_range(l, AUDIT_FIRST_ANOM_MSG, AUDIT_LAST_ANOM_MSG))
		sd.anomalies++;

	// add response to anomalies
	if (list_find_msg_range(l, AUDIT_FIRST_ANOM_RESP, AUDIT_LAST_ANOM_RESP))
		sd.responses++;
}

/*
 * This function compares strings. It returns a 0 if no match and a 1 if
 * there is a match 
 *
static int strmatch(const char *needle, const char *haystack)
{
	if (event_exact_match) {
		if (strcmp(haystack, needle) != 0)
			return 0; 
	} else {
		if (strstr(haystack, needle) == NULL)
			return 0; 
	}
	return 1;
} */

/*
 * This function compares user id's. It returns a 0 if no match and a 1 if
 * there is a match 
 *
static int user_match(llist *l)
{
	if (event_ua) {
		// This will "or" the user tests
		if (event_uid == l->s.uid)
			return 1;
		if (event_euid == l->s.euid)
			return 1;
		if (event_loginuid == l->s.loginuid)
			return 1;
		return 0;
	} else {
		// This will "and" the user tests
		if ((event_uid != -1) && (event_uid != l->s.uid))
			return 0;
		if ((event_euid != -1) &&(event_euid != l->s.euid))
			return 0;
		if ((event_loginuid != -1) &&
				(event_loginuid != l->s.loginuid))
			return 0;
	}
	return 1;
} */

/*
 * This function compares group id's. It returns a 0 if no match and a 1 if
 * there is a match 
 *
static int group_match(llist *l)
{
	if (event_ga) {
		// This will "or" the group tests
		if (event_gid == l->s.gid)
			return 1;
		if (event_egid == l->s.egid)
			return 1;
		return 0;
	} else {
		// This will "and" the group tests
		if ((event_gid != -1) && (event_gid != l->s.gid))
			return 0;
		if ((event_egid != -1) &&(event_egid != l->s.egid))
			return 0;
	}
	return 1;
}
*/
